/*
 * K-best measurements: Bryant and Hallaron
 *
 * Luis Paulo Santos 2006
 *
 */

#include <stdlib.h>
#include <stdio.h>

static int K, cnt;
static double tolerance;
static double *v;

void Kbest_set (int, double);
void Kbest_destroy (void);
void Kbest_ini (void);
int Kbest_insert (double);
double Kbest_get (void);

static void sort (void);
static double err (void);

void Kbest_set (int pK, double ptol) {
  K=pK;
  tolerance = ptol;
  v = (double *)malloc(K*sizeof(double));
  if (v==NULL)
    fprintf (stderr, "Kbest ERRO: memoria insuficiente\n");
}

void Kbest_destroy (void) {
  free (v);
}

void Kbest_ini (void) {
  cnt=0;
}

int Kbest_insert (double value) {
  if (cnt<K)
    v[cnt++] = value;
  else if (value < v[cnt-1])
    v[cnt-1] = value;

  sort ();

  if (err()<tolerance)
    return 1;
  else
    return 0;
}

double Kbest_get (void) {
  return (v[0]);
}

static void sort (void) {
  int i,j;
  double aux;

  for (i=0 ; i<(cnt-1) ; i++)
    for (j=i+1 ; j<cnt ; j++)
      if (v[i] > v[j]) {
	aux = v[i];
	v[i] = v[j];
	v[j] = aux;
      }
}

static double err (void) {
  return ((v[cnt-1]-v[0])/v[0]);
}


