

/*
 * Medicao do desempenho
 *
 * AC2 - LESI 2006
 *
 */

#include "clock.h"
#include "data.h"
#include "function.h"
#include "Kbest.h"
#include <stdio.h>
#include <string.h>

#define NBR_SIZES 14

static void ini_sizes (void);
static void ini_CPE (void);

static int sizes[NBR_SIZES];
static double CPE[NBR_SIZES];
static char str[100];

main () {
  int i,s, converge;
  double cycles;
  data_t result;

#ifdef OPER_MULT
  sprintf (str,"Product ");
#else
  sprintf (str,"Sum ");
#endif

#ifdef USE_FLOAT
  strcat (str, "with floats!...");
#else
  strcat (str, "with integers!...");
#endif

  // set Kbets parameters and alloc data structures
  Kbest_set (KBEST_K, KBEST_E);

  fprintf (stdout, "Starting %s\n", str);

  // iterating sizes
  ini_sizes ();
  ini_CPE ();
  for (s=0 ; s<NBR_SIZES ; s++) {

    //initialize data structure
    data_ini (sizes[s]);

    //warm_up
    f (&result);

    // restart Kbest counters
    Kbest_ini ();

    // iterations
    i=0;
    converge=0;
    while (!converge && i<KBEST_M) {
      start_counter ();
      // function to be measured
      f (&result);
      cycles = get_counter ();
      // insert into Kbest values
      converge = Kbest_insert (cycles/sizes[s]);
      converge = (i<KBEST_K ? 0 : converge);
      i++;
    }

    // get CPE for current size
    CPE[s] = Kbest_get();
    if (!converge)
      fprintf (stdout, "CPE did not converge!! ");
    fprintf (stdout, "%d elements\tCPE=%.2lf\n", sizes[s], CPE[s]);

    data_destroy ();

  } // end sizes loop

  // free Kbest data strucutes
  Kbest_destroy ();
  fprintf (stdout, "That's all, folks!!\n\n");
}

static void ini_sizes (void) {
  int i;

  for (i=0 ; i<NBR_SIZES ; i++)
    sizes[i] = 128 << i;
}

static void ini_CPE (void) {
  int i;

  for (i=0 ; i<NBR_SIZES ; i++)
    CPE[i] = 0.0L;
}

